package game;

import java.lang.reflect.Field;

import game.Textbox.Action;
import game.color.TextColor;

public class ColorVar extends IntVar {
	private final boolean alpha;
	
	public ColorVar(String name, String display, Field field, Object object, CVarCategory category, boolean alpha, IntFunction func) {
		super(name, display, field, object, category, Integer.MIN_VALUE, Integer.MAX_VALUE, func, null, 0);
		this.alpha = alpha;
	}

	public String getType() {
		return this.alpha ? (TextColor.GRAY + "color32") : (TextColor.LGRAY + "color");
	}
	
	protected Integer parseValue(String str) {
		if(str.length() != (this.alpha ? 8 : 6))
			return null;
		Integer value = Util.parseInt(str, -16);
		return this.alpha || (value & 0xff000000) == 0 ? (this.alpha ? value : (0xff000000 | value)) : null;
	}
	
	protected String formatValue(int value) {
		return String.format(this.alpha ? "%08x" : "%06x", this.alpha ? value : (value & 0x00ffffff));
	}
	
	public Slider selector(int x, int y, int w, int h) {
		throw new UnsupportedOperationException("Kann keinen Schieberegler für Farben erstellen");
	}
	
	public Label label(int x, int y, int w, int h) {
		return new Label(x, y, w, h, this.display);
	}
	
	public Textbox editor(int x, int y, int w, int h) {
		return new Textbox(x, y, w, h, this.alpha ? 8 : 6, true, new Textbox.Callback() {
			public void use(Textbox elem, Textbox.Action value) {
				if(value == Action.SEND || value == Action.UNFOCUS)
					ColorVar.this.parse(elem.getText());
			}
		}, this.format());
	}
}
