package game;

import game.Drawing.Vec2i;
import game.Dropdown.Handle;

public abstract class Element {
	protected final Game gm = Game.getGame();
	
	protected Gui gui;
	protected Formatter format;
	protected String text = "";
	
//	String format_text;
//	Object aux_data;
//	String[] format_data;
//	CVar cv_data;

	protected int pos_x;
	protected int pos_y;
	protected int size_x;
	protected int size_y;
	protected int text_x = 0;
	protected int text_y = 0;
	protected int tsize_x = 0;
	protected int tsize_y = 0;

	protected final int margin_x1 = this.getMargin();
	protected final int margin_y1 = this.getMargin();
	protected final int margin_x2 = this.getMargin();
	protected final int margin_y2 = this.getMargin();
	
	public boolean visible = true;
	public boolean enabled = true;
	public boolean r_dirty = true;

	
	public Element(int x, int y, int w, int h, Formatter formatter) {
		this.pos_x = x;
		this.pos_y = y;
		this.size_x = w;
		this.size_y = h;
		this.format = formatter;
//		gui_update_style(this, 1);
//		if(type != ElemType.SLIDER) {
//			this.margin_x1 = this.margin_x2 = (type == ElemType.FIELD || type == ElemType.DROPDOWN_HANDLE) ? this.border : 0;
//			this.margin_y1 = this.margin_y2 = (type == ElemType.FIELD || type == ElemType.DROPDOWN_HANDLE) ? this.border : 0;
//		}
	}
	
	public void setGui(Gui gui) {
		if(gui == null)
			throw new NullPointerException("gui");
		if(this.gui != null)
			throw new IllegalStateException("GUI bereits gesetzt");
		this.gui = gui;
	}
	
	protected int getMargin() {
		return 1;
	}
	
	public final int getX() {
		return this.pos_x;
	}
	
	public final int getY() {
		return this.pos_y;
	}
	
	public final int getWidth() {
		return this.size_x;
	}
	
	public final int getHeight() {
		return this.size_y;
	}
	
	protected boolean isTextCenteredX() {
		return true;
	}
	
	protected boolean isTextCenteredY() {
		return true;
	}
	
	protected boolean hasLinebreak() {
		return false;
	}
	
	public boolean canHover() {
		return true;
	}
	
	public boolean canClick() {
		return this.canHover();
	}

	public boolean inside(int x, int y) {
		return (x >= this.pos_x) && (x < (this.pos_x + this.size_x)) && (y >= this.pos_y) && (y < (this.pos_y + this.size_y));
	}

	public boolean intersects(Element elem) {
		return (elem.pos_y + elem.size_y) > this.pos_y && elem.pos_y < (this.pos_y + this.size_y) && (elem.pos_x + elem.size_x) > this.pos_x && elem.pos_x < (this.pos_x + this.size_x);
	}
	
	public void updateText() {
		Vec2i size = Drawing.txt_size(this.pos_x + this.margin_x1, this.pos_y + this.margin_y1, 
				this.pos_x + this.margin_x1, this.pos_y + this.margin_y1, 
				this.hasLinebreak() ? (this.pos_x + (this.size_x - (this.margin_x1 + this.margin_x2))) : Integer.MAX_VALUE, Integer.MAX_VALUE, this.text);
		this.tsize_x = size.xpos;
		this.tsize_y = size.ypos;
//		if(this.type != ElemType.FIELD) {
		if(this.isTextCenteredX())
			this.text_x = (this.size_x - this.tsize_x) / 2;
		if(this.isTextCenteredY())
			this.text_y = (this.size_y - this.tsize_y - Font.YGLYPH) / 2;
//		}
		// logd("DBG", "s = %d %d; o = %d %d", this.tsize_x, this.tsize_y, this.text_x, this.text_y);
//			gui_update_dropdown();
		if(this.gui != null && this.gui.selected instanceof Handle && this.gui.selected.visible)
			this.gui.selected.r_dirty = true;
		this.r_dirty = true;
	}
	
	public void setText(String str) {
		this.text = str;
		this.updateText();
	}
	
	public String getText() {
		return this.text;
	}
	
	protected void formatText() {
		if(this.format != null) {
			this.text = this.format.use(this);
			this.updateText();
		}
	}
	
	public void scroll(int scr_x, int scr_y, int x, int y, boolean ctrl, boolean shift) {
		
	}
	
	public void mouse(Button btn, int x, int y, boolean ctrl, boolean shift) {
		
	}
	
	public void mouserel() {
		
	}
	
	public void drag(int x, int y) {
		
	}
	
	public void character(char code) {
		
	}
	
	public void key(Keysym key, boolean ctrl, boolean shift) {
		
	}
	
	public void select() {
		
	}
	
	public void deselect() {
		
	}
	
	public void update() {
		
	}
	
	public void setSelected() {
		if(this.gui != null)
			this.gui.select(this);
	}
	
	public void setDeselected() {
		if(this.gui != null && this.gui.selected == this)
			this.gui.deselect();
	}
	
	public void shift(int shift_x, int shift_y) {
		this.pos_x += shift_x;
		this.pos_y += shift_y;
	}
	
	public void reformat() {
		if(this.format != null) {
//			boolean flag = this.r_dirty;
			this.formatText();
//			if(!flag && !this.visible)
//				this.r_dirty = false;
		}
	}
	
	protected void drawBackground() {
		Drawing.drawGradient2Border(this.pos_x, this.pos_y, this.size_x, this.size_y, this.gm.style.fill_top, this.gm.style.fill_btm, this.gm.style.brdr_top, this.gm.style.brdr_btm);
//		Drawing.drawGradient2GradBorder(this.pos_x, this.pos_y, this.size_x, this.size_y, this.gm.style.fill_top, this.gm.style.fill_btm, 0xff000000, 
//				this.gm.style.brdr_top, Drawing.mixColor(this.gm.style.brdr_top, this.gm.style.brdr_btm), Drawing.mixColor(this.gm.style.brdr_top, this.gm.style.brdr_btm), this.gm.style.brdr_btm);
	}
	
	protected void drawForeground(int x1, int y1, int x2, int y2) {
		Drawing.drawText(this.text, x1 + this.text_x, y1 + this.text_y, this.gm.style.text_base);
	}
	
	public void draw() {
//		if(this.r_dirty) {
			if(this.visible) {
//				WCF.glScissor(this.pos_x, (this.gm.fb_y - (this.pos_y + this.size_y)) < 0 ? 0 : (this.gm.fb_y - (this.pos_y + this.size_y)), this.size_x, this.size_y);
//				WCF.glEnable(WCF.GL_SCISSOR_TEST);
//				WCF.glClear(WCF.GL_COLOR_BUFFER_BIT);
//				WCF.glDisable(WCF.GL_SCISSOR_TEST);
				this.drawBackground();
//				if(this.selected == this && this.type == ElemType.DROPDOWN_HANDLE)
//				continue;
				int x1 = this.pos_x + this.margin_x1;
				int y1 = this.pos_y + this.margin_y1;
				int x2 = this.size_x - (this.margin_x1 + this.margin_x2);
				int y2 = this.size_y - (this.margin_y1 + this.margin_y2);
				// if(elem.type == ElemType.FIELD) {
				WCF.glScissor(x1 < 0 ? 0 : x1, (this.gm.fb_y - (y1 + y2)) < 0 ? 0 : (this.gm.fb_y - (y1 + y2)), x2 < 0 ? 0 : x2, y2 < 0 ? 0 : y2);
				WCF.glEnable(WCF.GL_SCISSOR_TEST);
				// }
	//			if(this.type == ElemType.CUSTOM)
	//				this.func(this, 1);
	//			else
				this.drawForeground(x1, y1, x2, y2);
				// logd("DBG", "%d @ %d %d -> %d %d", elem.id, x1, y1, elem.pos_x + x2, elem.pos_y + y2);
				// if(elem.type == ElemType.FIELD) {
				WCF.glDisable(WCF.GL_SCISSOR_TEST);
					// glScissor(0, 0, sys.fb_x, sys.fb_y);
				// }
			}
//			else {
//				WCF.glScissor(this.pos_x, (this.gm.fb_y - (this.pos_y + this.size_y)) < 0 ? 0 : (this.gm.fb_y - (this.pos_y + this.size_y)), this.size_x, this.size_y);
//				// logd("DBG", "%d @ %d %d -> %d %d", elem.id, elem.pos_x, elem.pos_y, elem.pos_x + elem.size_x, elem.pos_y + elem.size_y);
//				WCF.glEnable(WCF.GL_SCISSOR_TEST);
////				if(this.type == ElemType.CUSTOM)
////					this.func(this, 0);
////				else
//				WCF.glClear(WCF.GL_COLOR_BUFFER_BIT);
//				WCF.glDisable(WCF.GL_SCISSOR_TEST);
//				this.r_dirty = false;
//			}
			// this.r_dirty = this.visible;
			// logd("DBG", "@ r");
//			if(this.visible) {
//			}
//			else {
//			}
//		}
	}
	
	public void drawOverlay() {
		
	}
	
	public void drawHover() {
		Drawing.drawRectColor(this.pos_x, this.pos_y, this.size_x, this.size_y, this.gm.style.hover);
	}
	
	public void drawPress() {
		Drawing.drawRectColor(this.pos_x, this.pos_y, this.size_x, this.size_y, this.gm.style.press);
	}
}
