package game;

import game.Variable.IntType;
import game.properties.IStringSerializable;

public enum Style implements IStringSerializable, Displayable {
	DEFAULT("default", "Glänzend (Standard)"), GRAY("gray", "Grau"), BLUE("blue", "Blau"), CUSTOM("custom", "Angepasst");

	public final String id;
	public final String name;
	
	private Style(String id, String name) {
		this.id = id;
		this.name = name;
	}

	@Variable(type = IntType.COLOR, name = "color_border_top", category = CVarCategory.STYLE, display = "Umrahmung")
	public int brdr_top;
	@Variable(type = IntType.COLOR, name = "color_border_btm", category = CVarCategory.STYLE)
	public int brdr_btm;

	@Variable(type = IntType.COLOR, name = "color_button_top", category = CVarCategory.STYLE, display = "Knopf")
	public int fill_top;
	@Variable(type = IntType.COLOR, name = "color_button_btm", category = CVarCategory.STYLE)
	public int fill_btm;
	@Variable(type = IntType.COLOR, name = "color_textbox_top", category = CVarCategory.STYLE, display = "Textfeld")
	public int field_top;
	@Variable(type = IntType.COLOR, name = "color_textbox_btm", category = CVarCategory.STYLE)
	public int field_btm;

	@Variable(type = IntType.COLOR, name = "color_label_text", category = CVarCategory.STYLE, display = "Beschriftung")
	public int text_label;
	@Variable(type = IntType.COLOR, name = "color_button_text", category = CVarCategory.STYLE, display = "Text Knopf")
	public int text_base;
	@Variable(type = IntType.COLOR, name = "color_textbox_text", category = CVarCategory.STYLE, display = "Textfeld Text")
	public int text_field;

	@Variable(type = IntType.COLOR, name = "color_background_t", category = CVarCategory.STYLE, display = "Hintergrund")
	public int bg_top;
	@Variable(type = IntType.COLOR, name = "color_background_b", category = CVarCategory.STYLE)
	public int bg_btm;

	@Variable(type = IntType.ALPHA, name = "color_press", category = CVarCategory.STYLE, display = "Gedrückt")
	public int press;
	@Variable(type = IntType.ALPHA, name = "color_hover", category = CVarCategory.STYLE, display = "Gewählt")
	public int hover;
	@Variable(type = IntType.ALPHA, name = "color_select", category = CVarCategory.STYLE, display = "Textauswahl")
	public int select;
	@Variable(type = IntType.ALPHA, name = "color_cursor", category = CVarCategory.STYLE, display = "Textmarke")
	public int cursor;
	
	static {
		DEFAULT
			.border(0xffffff, 0x3f3f3f)
			.background(0x000000, 0x000000)
			.base(0x404040, 0x000000, 0xffffff, 0xefefef)
			.field(0x000000, 0x202020, 0xdfdfdf)
			.select(0x30ffffff, 0x28ffffff, 0x60dfdfdf, 0xffffffff);
		
		GRAY
			.border(0x000000, 0x202020)
			.background(0x404040, 0x0a0a0a)
			.base(0x808080, 0x000000, 0xffffff, 0xffffff)
			.field(0x404040, 0x808080, 0xffffff)
			.select(0x18ffffff, 0x288080ff, 0x808080ff, 0xff000000);
	
		BLUE
			.border(0x0000df, 0x300020)
			.background(0x20208f, 0x0a0a2d)
			.base(0x2020a0, 0x000020, 0x8fffaf, 0x00cfaf)
			.field(0x505090, 0x406060, 0xcfdfff)
			.select(0x288f00ff, 0x28c080ff, 0x604020ff, 0xff2fff6f);
		
		CUSTOM
			.border(0x000000, 0x000000)
			.background(0x404040, 0x404040)
			.base(0x808080, 0x808080, 0xffffff, 0xffffff)
			.field(0x808080, 0x808080, 0xffffff)
			.select(0x18ffffff, 0x288080ff, 0x808080ff, 0xff000000);
	}

	private Style border(int top, int btm) {
		this.brdr_top = top | 0xff000000;
		this.brdr_btm = btm | 0xff000000;
		return this;
	}
	
	private Style background(int top, int btm) {
		this.bg_top = top | 0xff000000;
		this.bg_btm = btm | 0xff000000;
		return this;
	}
	
	private Style base(int top, int btm, int text, int label) {
		this.fill_top = top | 0xff000000;
		this.fill_btm = btm | 0xff000000;
		this.text_base = text | 0xff000000;
		this.text_label = label | 0xff000000;
		return this;
	}
	
	private Style field(int top, int btm, int text) {
		this.field_top = top | 0xff000000;
		this.field_btm = btm | 0xff000000;
		this.text_field = text | 0xff000000;
		return this;
	}
	
	private Style select(int prs, int hov, int sel, int cur) {
		this.press = prs;
		this.hover = hov;
		this.select = sel;
		this.cursor = cur;
		return this;
	}
	
	public String getName() {
		return this.id;
	}
	
	public String getDisplay() {
		return this.name;
	}
}
