package game.entity.projectile;

import java.util.List;

import game.ExtMath;
import game.enchantment.EnchantmentHelper;
import game.entity.DamageSource;
import game.entity.Entity;
import game.entity.types.EntityLiving;
import game.entity.types.IObjectData;
import game.entity.types.IProjectile;
import game.init.Config;
import game.init.SoundEvent;
import game.nbt.NBTTagCompound;
import game.world.BoundingBox;
import game.world.HitPosition;
import game.world.Vec3;
import game.world.World;

public class EntityBullet extends Entity implements IProjectile, IObjectData
{
    public Entity shootingEntity;
    private int ticksInAir;
    private int age;
    private int damage = 5;

    public EntityBullet(World worldIn)
    {
        super(worldIn);
        this.renderDistWeight = 10.0D;
        this.setSize(0.5F, 0.5F);
    }

    public EntityBullet(World worldIn, double x, double y, double z)
    {
        super(worldIn);
        this.renderDistWeight = 10.0D;
        this.setSize(0.5F, 0.5F);
        this.setPosition(x, y, z);
    }

    public EntityBullet(World worldIn, double x, double y, double z, int data)
    {
    	this(worldIn, x, y, z);
    	Entity entity2 = worldIn.getEntityByID(data);
        if(entity2 instanceof EntityLiving) {
            this.shootingEntity = entity2;
        }
    }

    public EntityBullet(World worldIn, EntityLiving shooter, EntityLiving target, float velocity, float innacuracy)
    {
        super(worldIn);
        this.renderDistWeight = 10.0D;
        this.shootingEntity = shooter;

        this.posY = shooter.posY + (double)shooter.getEyeHeight() - 0.10000000149011612D;
        double d0 = target.posX - shooter.posX;
        double d1 = target.getEntityBoundingBox().minY + (double)(target.height * 0.7f) - this.posY;
        double d2 = target.posZ - shooter.posZ;
        double d3 = (double)ExtMath.sqrtd(d0 * d0 + d2 * d2);

        if (d3 >= 1.0E-7D)
        {
            float f = (float)(ExtMath.atan2(d2, d0) * 180.0D / Math.PI) - 90.0F;
            float f1 = (float)(-(ExtMath.atan2(d1, d3) * 180.0D / Math.PI));
            double d4 = d0 / d3;
            double d5 = d2 / d3;
            this.setLocationAndAngles(shooter.posX + d4, this.posY, shooter.posZ + d5, f, f1);
//            float f2 = (float)(d3 * 0.20000000298023224D);
            this.setThrowableHeading(d0, d1 /* + (double)f2 */ , d2, velocity, innacuracy);
        }
    }

    public EntityBullet(World worldIn, EntityLiving shooter, float velocity)
    {
        super(worldIn);
        this.renderDistWeight = 10.0D;
        this.shootingEntity = shooter;

        this.setSize(0.5F, 0.5F);
        this.setLocationAndAngles(shooter.posX, shooter.posY + (double)shooter.getEyeHeight(), shooter.posZ, shooter.rotYaw, shooter.rotPitch);
        this.posX -= (double)(ExtMath.cos(this.rotYaw / 180.0F * (float)Math.PI) * 0.16F);
        this.posY -= 0.10000000149011612D;
        this.posZ -= (double)(ExtMath.sin(this.rotYaw / 180.0F * (float)Math.PI) * 0.16F);
        this.setPosition(this.posX, this.posY, this.posZ);
        this.motionX = (double)(-ExtMath.sin(this.rotYaw / 180.0F * (float)Math.PI) * ExtMath.cos(this.rotPitch / 180.0F * (float)Math.PI));
        this.motionZ = (double)(ExtMath.cos(this.rotYaw / 180.0F * (float)Math.PI) * ExtMath.cos(this.rotPitch / 180.0F * (float)Math.PI));
        this.motionY = (double)(-ExtMath.sin(this.rotPitch / 180.0F * (float)Math.PI));
        this.setThrowableHeading(this.motionX, this.motionY, this.motionZ, velocity, 0.5F);
    }

    protected void entityInit()
    {
    }

    /**
     * Similar to setArrowHeading, it's point the throwable entity to a x, y, z direction.
     */
    public void setThrowableHeading(double x, double y, double z, float velocity, float inaccuracy)
    {
        float f = ExtMath.sqrtd(x * x + y * y + z * z);
        x = x / (double)f;
        y = y / (double)f;
        z = z / (double)f;
        x = x + this.rand.gaussian() * (double)(this.rand.chance() ? -1 : 1) * 0.007499999832361937D * (double)inaccuracy;
        y = y + this.rand.gaussian() * (double)(this.rand.chance() ? -1 : 1) * 0.007499999832361937D * (double)inaccuracy;
        z = z + this.rand.gaussian() * (double)(this.rand.chance() ? -1 : 1) * 0.007499999832361937D * (double)inaccuracy;
        x = x * (double)velocity;
        y = y * (double)velocity;
        z = z * (double)velocity;
        this.motionX = x;
        this.motionY = y;
        this.motionZ = z;
        float f1 = ExtMath.sqrtd(x * x + z * z);
        this.prevYaw = this.rotYaw = (float)(ExtMath.atan2(x, z) * 180.0D / Math.PI);
        this.prevPitch = this.rotPitch = (float)(ExtMath.atan2(y, (double)f1) * 180.0D / Math.PI);
    }

    public void setPositionAndRotation2(double x, double y, double z, float yaw, float pitch, int posRotationIncrements, boolean p_180426_10_)
    {
        this.setPosition(x, y, z);
        this.setRotation(yaw, pitch);
    }

    /**
     * Sets the velocity to the args. Args: x, y, z
     */
    public void setVelocity(double x, double y, double z)
    {
        this.motionX = x;
        this.motionY = y;
        this.motionZ = z;

        if (this.prevPitch == 0.0F && this.prevYaw == 0.0F)
        {
            float f = ExtMath.sqrtd(x * x + z * z);
            this.prevYaw = this.rotYaw = (float)(ExtMath.atan2(x, z) * 180.0D / Math.PI);
            this.prevPitch = this.rotPitch = (float)(ExtMath.atan2(y, (double)f) * 180.0D / Math.PI);
            this.prevPitch = this.rotPitch;
            this.prevYaw = this.rotYaw;
            this.setLocationAndAngles(this.posX, this.posY, this.posZ, this.rotYaw, this.rotPitch);
        }
    }

    /**
     * Called to update the entity's position/logic.
     */
    public void onUpdate()
    {
        super.onUpdate();

        if (this.prevPitch == 0.0F && this.prevYaw == 0.0F)
        {
            float f = ExtMath.sqrtd(this.motionX * this.motionX + this.motionZ * this.motionZ);
            this.prevYaw = this.rotYaw = (float)(ExtMath.atan2(this.motionX, this.motionZ) * 180.0D / Math.PI);
            this.prevPitch = this.rotPitch = (float)(ExtMath.atan2(this.motionY, (double)f) * 180.0D / Math.PI);
        }

        if(!this.worldObj.client && ++this.age >= 200) {
        	this.setDead();
        	return;
        }
        ++this.ticksInAir;
        Vec3 vec31 = new Vec3(this.posX, this.posY, this.posZ);
        Vec3 vec3 = new Vec3(this.posX + this.motionX, this.posY + this.motionY, this.posZ + this.motionZ);
        HitPosition movingobjectposition = this.worldObj.rayTraceBlocks(vec31, vec3, false, true, false);
        vec31 = new Vec3(this.posX, this.posY, this.posZ);
        vec3 = new Vec3(this.posX + this.motionX, this.posY + this.motionY, this.posZ + this.motionZ);

        if (movingobjectposition != null)
        {
            vec3 = new Vec3(movingobjectposition.vec.xCoord, movingobjectposition.vec.yCoord, movingobjectposition.vec.zCoord);
        }

        Entity entity = null;
        List<Entity> list = this.worldObj.getEntitiesWithinAABBExcludingEntity(this, this.getEntityBoundingBox().addCoord(this.motionX, this.motionY, this.motionZ).expand(1.0D, 1.0D, 1.0D));
        double d0 = 0.0D;

        for (int i = 0; i < list.size(); ++i)
        {
            Entity entity1 = (Entity)list.get(i);

            if (entity1.canBeCollidedWith() && (entity1 != this.shootingEntity || this.ticksInAir >= 5))
            {
                float f1 = 0.3F;
                BoundingBox axisalignedbb1 = entity1.getEntityBoundingBox().expand((double)f1, (double)f1, (double)f1);
                HitPosition movingobjectposition1 = axisalignedbb1.calculateIntercept(vec31, vec3);

                if (movingobjectposition1 != null)
                {
                    double d1 = vec31.squareDistanceTo(movingobjectposition1.vec);

                    if (d1 < d0 || d0 == 0.0D)
                    {
                        entity = entity1;
                        d0 = d1;
                    }
                }
            }
        }

        if (entity != null)
        {
            movingobjectposition = new HitPosition(entity);
        }

//        if (movingobjectposition != null && movingobjectposition.entity != null && movingobjectposition.entity.isPlayer())
//        {
//            EntityNPC entityplayer = (EntityNPC)movingobjectposition.entity;
//
//            if (entityplayer.creative || this.shootingEntity.isPlayer() && !((EntityNPC)this.shootingEntity).canAttackPlayer(entityplayer))
//            {
//                movingobjectposition = null;
//            }
//        }

        if (movingobjectposition != null)
        {
            if (movingobjectposition.entity != null)
            {
                float f2 = ExtMath.sqrtd(this.motionX * this.motionX + this.motionY * this.motionY + this.motionZ * this.motionZ);
                int l = ExtMath.ceild((double)f2 * (double)this.damage);

                DamageSource damagesource;

                if (this.shootingEntity == null)
                {
                    damagesource = DamageSource.causeShotDamage(this, this);
                }
                else
                {
                    damagesource = DamageSource.causeShotDamage(this, this.shootingEntity);
                }

                if ((this.worldObj.client || Config.damageBullet) && movingobjectposition.entity.attackEntityFrom(damagesource, l))
                {
                    if (movingobjectposition.entity instanceof EntityLiving)
                    {
                        EntityLiving entitylivingbase = (EntityLiving)movingobjectposition.entity;

//                        if (!this.worldObj.client)
//                        {
//                            entitylivingbase.setArrowCountInEntity(entitylivingbase.getArrowCountInEntity() + 1);
//                        }

//                        if (this.knockbackStrength > 0)
//                        {
//                            float f7 = MathHelper.sqrt_double(this.motionX * this.motionX + this.motionZ * this.motionZ);
//
//                            if (f7 > 0.0F)
//                            {
//                                movingobjectposition.entityHit.addKnockback(this.motionX * (double)this.knockbackStrength * 0.6000000238418579D / (double)f7, 0.1D, this.motionZ * (double)this.knockbackStrength * 0.6000000238418579D / (double)f7);
//                            }
//                        }

                        if (this.shootingEntity instanceof EntityLiving)
                        {
                            EnchantmentHelper.applyThornEnchantments(entitylivingbase, this.shootingEntity);
                            EnchantmentHelper.applyArthropodEnchantments((EntityLiving)this.shootingEntity, entitylivingbase);
                        }

//                            if (this.shootingEntity != null && movingobjectposition.entityHit != this.shootingEntity && movingobjectposition.entityHit.isPlayer() && this.shootingEntity.isPlayer())
//                            {
//                                ((EntityNPCMP)this.shootingEntity).netHandler.sendPacket(new S29PacketSoundEffect(
//                                		"random.successful_hit", shootingEntity.posX, 
//                                		shootingEntity.posY + (double)shootingEntity.getEyeHeight(), shootingEntity.posZ, 0.18F, 0.45F));
//                            }
                    }

                    this.playSound(SoundEvent.METALHIT, 0.2F, this.rand.floatv() * 0.2F + 1.5F);
                    
//                    this.setDead();
                }
//                else
//                {
                this.setDead();
//                    this.motionX *= -0.10000000149011612D;
//                    this.motionY *= -0.10000000149011612D;
//                    this.motionZ *= -0.10000000149011612D;
//                    this.rotYaw += 180.0F;
//                    this.prevYaw += 180.0F;
//                    this.ticksInAir = 0;
//                }
            }
            else
            {
                this.playSound(SoundEvent.METALHIT, 0.5F, this.rand.floatv() * 0.2F + 1.5F);
                
            	this.setDead();
            }
        }

        this.posX += this.motionX;
        this.posY += this.motionY;
        this.posZ += this.motionZ;
        float f3 = ExtMath.sqrtd(this.motionX * this.motionX + this.motionZ * this.motionZ);
        this.rotYaw = (float)(ExtMath.atan2(this.motionX, this.motionZ) * 180.0D / Math.PI);

        for (this.rotPitch = (float)(ExtMath.atan2(this.motionY, (double)f3) * 180.0D / Math.PI); this.rotPitch - this.prevPitch < -180.0F; this.prevPitch -= 360.0F)
        {
            ;
        }

        while (this.rotPitch - this.prevPitch >= 180.0F)
        {
            this.prevPitch += 360.0F;
        }

        while (this.rotYaw - this.prevYaw < -180.0F)
        {
            this.prevYaw -= 360.0F;
        }

        while (this.rotYaw - this.prevYaw >= 180.0F)
        {
            this.prevYaw += 360.0F;
        }

        this.rotPitch = this.prevPitch + (this.rotPitch - this.prevPitch) * 0.2F;
        this.rotYaw = this.prevYaw + (this.rotYaw - this.prevYaw) * 0.2F;
//        float f4 = 0.99F;
//        float f6 = 0.001F;

//        if (this.isInLiquid())
//        {
//            for (int i1 = 0; i1 < 4; ++i1)
//            {
//                float f8 = 0.25F;
//                this.worldObj.spawnParticle(ParticleType.WATER_BUBBLE, this.posX - this.motionX * (double)f8, this.posY - this.motionY * (double)f8, this.posZ - this.motionZ * (double)f8, this.motionX, this.motionY, this.motionZ);
//            }
//
//            f4 = 0.85F;
//        }

//        if (this.isWet())
//        {
//            this.extinguish();
//        }

//        this.motionX *= (double)f4;
//        this.motionY *= (double)f4;
//        this.motionZ *= (double)f4;
//        this.motionY -= (double)f6;
        this.setPosition(this.posX, this.posY, this.posZ);
//        this.doBlockCollisions();
    }
    
    public void writeEntityToNBT(NBTTagCompound tagCompound)
    {
        tagCompound.setInteger("damage", this.damage);
        tagCompound.setInteger("age", this.age);
    }
    
    public void readEntityFromNBT(NBTTagCompound tagCompund)
    {
        if(tagCompund.hasKey("damage", 99))
            this.damage = tagCompund.getInteger("damage");
        this.age = tagCompund.getInteger("age");
    }
    
    protected boolean canTriggerWalking()
    {
        return false;
    }

    public void setDamage(int damageIn)
    {
        this.damage = damageIn;
    }

    public int getDamage()
    {
        return this.damage;
    }
    
    public boolean canAttackWithItem()
    {
        return false;
    }

    public float getEyeHeight()
    {
        return 0.0F;
    }
    
    public int getTrackingRange() {
    	return 96;
    }
    
    public int getUpdateFrequency() {
    	return 1;
    }
    
    public boolean isSendingVeloUpdates() {
    	return false;
    }
	
	public int getPacketData() {
		return this.shootingEntity != null ? this.shootingEntity.getId() : this.getId();
	}
	
	public boolean hasSpawnVelocity() {
		return true;
	}
}
