package game.renderer.entity;

import game.WCF;
import game.entity.projectile.EntityBullet;
import game.renderer.DefaultVertexFormats;
import game.renderer.GlState;
import game.renderer.RenderBuffer;
import game.renderer.Tessellator;


public class RenderBullet extends Render<EntityBullet>
{
    private static final String bulletTextures = "textures/entity/bullet.png";

    public RenderBullet(RenderManager renderManagerIn)
    {
        super(renderManagerIn);
    }
    
    public void doRender(EntityBullet entity, double x, double y, double z, float partialTicks)
    {
        this.bindEntityTexture(entity);
        GlState.color(1.0F, 1.0F, 1.0F, 1.0F);
        WCF.glPushMatrix();
        WCF.glTranslatef((float)x, (float)y, (float)z);
        WCF.glRotatef(entity.prevYaw + (entity.rotYaw - entity.prevYaw) * partialTicks - 90.0F, 0.0F, 1.0F, 0.0F);
        WCF.glRotatef(entity.prevPitch + (entity.rotPitch - entity.prevPitch) * partialTicks, 0.0F, 0.0F, 1.0F);
        RenderBuffer worldrenderer = Tessellator.getBuffer();
        float f = 0.0F;
        float f1 = 0.5F;
        float f2 = (float)0 / 32.0F;
        float f3 = (float)5 / 32.0F;
        float f4 = 0.0F;
        float f5 = 0.15625F;
        float f6 = (float)5 / 32.0F;
        float f7 = (float)10 / 32.0F;
        float f8 = 0.05625F;
        GlState.enableRescaleNormal();

        WCF.glRotatef(45.0F, 1.0F, 0.0F, 0.0F);
        WCF.glScalef(f8, f8, f8);
        WCF.glTranslatef(-4.0F, 0.0F, 0.0F);
        WCF.glNormal3f(f8, 0.0F, 0.0F);
        worldrenderer.begin(7, DefaultVertexFormats.POSITION_TEX);
        worldrenderer.pos(-7.0D, -2.0D, -2.0D).tex((double)f4, (double)f6).endVertex();
        worldrenderer.pos(-7.0D, -2.0D, 2.0D).tex((double)f5, (double)f6).endVertex();
        worldrenderer.pos(-7.0D, 2.0D, 2.0D).tex((double)f5, (double)f7).endVertex();
        worldrenderer.pos(-7.0D, 2.0D, -2.0D).tex((double)f4, (double)f7).endVertex();
        Tessellator.draw();
        WCF.glNormal3f(-f8, 0.0F, 0.0F);
        worldrenderer.begin(7, DefaultVertexFormats.POSITION_TEX);
        worldrenderer.pos(-7.0D, 2.0D, -2.0D).tex((double)f4, (double)f6).endVertex();
        worldrenderer.pos(-7.0D, 2.0D, 2.0D).tex((double)f5, (double)f6).endVertex();
        worldrenderer.pos(-7.0D, -2.0D, 2.0D).tex((double)f5, (double)f7).endVertex();
        worldrenderer.pos(-7.0D, -2.0D, -2.0D).tex((double)f4, (double)f7).endVertex();
        Tessellator.draw();

        for (int j = 0; j < 4; ++j)
        {
            WCF.glRotatef(90.0F, 1.0F, 0.0F, 0.0F);
            WCF.glNormal3f(0.0F, 0.0F, f8);
            worldrenderer.begin(7, DefaultVertexFormats.POSITION_TEX);
            worldrenderer.pos(-8.0D, -2.0D, 0.0D).tex((double)f, (double)f2).endVertex();
            worldrenderer.pos(8.0D, -2.0D, 0.0D).tex((double)f1, (double)f2).endVertex();
            worldrenderer.pos(8.0D, 2.0D, 0.0D).tex((double)f1, (double)f3).endVertex();
            worldrenderer.pos(-8.0D, 2.0D, 0.0D).tex((double)f, (double)f3).endVertex();
            Tessellator.draw();
        }

        GlState.disableRescaleNormal();
        WCF.glPopMatrix();
        super.doRender(entity, x, y, z, partialTicks);
    }
    
    protected String getEntityTexture(EntityBullet entity)
    {
        return bulletTextures;
    }
}
