package game.renderer.tileentity;

import game.WCF;
import game.block.Block;
import game.block.BlockChest;
import game.renderer.GlState;
import game.renderer.model.ModelChest;
import game.renderer.model.ModelLargeChest;
import game.tileentity.TileEntityChest;


public class TileEntityChestRenderer extends TileEntitySpecialRenderer<TileEntityChest>
{
    private static final String textureTrappedDouble = "textures/blocks/chest_trapped_double.png";
//    private static final ResourceLocation textureChristmasDouble = new ResourceLocation("textures/tile/chest_christmas_double.png");
    private static final String textureNormalDouble = "textures/blocks/chest_normal_double.png";
    private static final String textureTrapped = "textures/blocks/chest_trapped.png";
//    private static final ResourceLocation textureChristmas = new ResourceLocation("textures/tile/chest_christmas.png");
    private static final String textureNormal = "textures/blocks/chest_normal.png";
    
    private ModelChest simpleChest = new ModelChest();
    private ModelChest largeChest = new ModelLargeChest();
//    private boolean isChristmas;

//    public TileEntityChestRenderer()
//    {
//        this.setChristmas(true);
//    }
    
//    public void setChristmas(boolean xmas) {
//    	if(!xmas) {
//    		this.isChristmas = false;
//    	}
//    	else {
//	    	Calendar calendar = Calendar.getInstance();
//	        if(calendar.get(2) + 1 == 12 && calendar.get(5) >= 24 && calendar.get(5) <= 26)
//	            this.isChristmas = true;
//    	}
//    }

    public void renderTileEntityAt(TileEntityChest te, double x, double y, double z, float partialTicks, int destroyStage)
    {
        GlState.enableDepth();
        GlState.depthFunc(515);
        GlState.depthMask(true);
        int i;

        if (!te.hasWorldObj())
        {
            i = 0;
        }
        else
        {
            Block block = te.getBlockType();
            i = te.getBlockMetadata();

            if (block instanceof BlockChest && i == 0)
            {
                ((BlockChest)block).checkForSurroundingChests(te.getWorld(), te.getPos(), te.getWorld().getState(te.getPos()));
                i = te.getBlockMetadata();
            }

            te.checkForAdjacentChests();
        }

        if (te.adjacentChestZNeg == null && te.adjacentChestXNeg == null)
        {
            ModelChest modelchest;

            if (te.adjacentChestXPos == null && te.adjacentChestZPos == null)
            {
                modelchest = this.simpleChest;

                if (destroyStage >= 0)
                {
                    this.bindTexture(DESTROY_STAGES[destroyStage]);
                    WCF.glMatrixMode(5890);
                    WCF.glPushMatrix();
                    WCF.glScalef(4.0F, 4.0F, 1.0F);
                    WCF.glTranslatef(0.0625F, 0.0625F, 0.0625F);
                    WCF.glMatrixMode(5888);
                }
//                else if (this.isChristmas)
//                {
//                    this.bindTexture(textureChristmas);
//                }
                else if (te.getChestType() == 1)
                {
                    this.bindTexture(textureTrapped);
                }
                else
                {
                    this.bindTexture(textureNormal);
                }
            }
            else
            {
                modelchest = this.largeChest;

                if (destroyStage >= 0)
                {
                    this.bindTexture(DESTROY_STAGES[destroyStage]);
                    WCF.glMatrixMode(5890);
                    WCF.glPushMatrix();
                    WCF.glScalef(8.0F, 4.0F, 1.0F);
                    WCF.glTranslatef(0.0625F, 0.0625F, 0.0625F);
                    WCF.glMatrixMode(5888);
                }
//                else if (this.isChristmas)
//                {
//                    this.bindTexture(textureChristmasDouble);
//                }
                else if (te.getChestType() == 1)
                {
                    this.bindTexture(textureTrappedDouble);
                }
                else
                {
                    this.bindTexture(textureNormalDouble);
                }
            }

            WCF.glPushMatrix();
            GlState.enableRescaleNormal();

            if (destroyStage < 0)
            {
                GlState.color(1.0F, 1.0F, 1.0F, 1.0F);
            }

            WCF.glTranslatef((float)x, (float)y + 1.0F, (float)z + 1.0F);
            WCF.glScalef(1.0F, -1.0F, -1.0F);
            WCF.glTranslatef(0.5F, 0.5F, 0.5F);
            int j = 0;

            if (i == 2)
            {
                j = 180;
            }

            if (i == 3)
            {
                j = 0;
            }

            if (i == 4)
            {
                j = 90;
            }

            if (i == 5)
            {
                j = -90;
            }

            if (i == 2 && te.adjacentChestXPos != null)
            {
                WCF.glTranslatef(1.0F, 0.0F, 0.0F);
            }

            if (i == 5 && te.adjacentChestZPos != null)
            {
                WCF.glTranslatef(0.0F, 0.0F, -1.0F);
            }

            WCF.glRotatef((float)j, 0.0F, 1.0F, 0.0F);
            WCF.glTranslatef(-0.5F, -0.5F, -0.5F);
            float f = te.prevLidAngle + (te.lidAngle - te.prevLidAngle) * partialTicks;

            if (te.adjacentChestZNeg != null)
            {
                float f1 = te.adjacentChestZNeg.prevLidAngle + (te.adjacentChestZNeg.lidAngle - te.adjacentChestZNeg.prevLidAngle) * partialTicks;

                if (f1 > f)
                {
                    f = f1;
                }
            }

            if (te.adjacentChestXNeg != null)
            {
                float f2 = te.adjacentChestXNeg.prevLidAngle + (te.adjacentChestXNeg.lidAngle - te.adjacentChestXNeg.prevLidAngle) * partialTicks;

                if (f2 > f)
                {
                    f = f2;
                }
            }

            f = 1.0F - f;
            f = 1.0F - f * f * f;
            modelchest.chestLid.rotateAngleX = -(f * (float)Math.PI / 2.0F);
            modelchest.renderAll();
            GlState.disableRescaleNormal();
            WCF.glPopMatrix();
            GlState.color(1.0F, 1.0F, 1.0F, 1.0F);

            if (destroyStage >= 0)
            {
                WCF.glMatrixMode(5890);
                WCF.glPopMatrix();
                WCF.glMatrixMode(5888);
            }
        }
    }
}
