package game.worldgen.feature;

import game.ExtMath;
import game.block.Block;
import game.init.Blocks;
import game.material.Material;
import game.rng.Random;
import game.world.BlockPos;
import game.world.WorldServer;
import game.worldgen.FeatureGenerator;

public class WorldGenIceSpike extends FeatureGenerator
{
    public boolean generate(WorldServer worldIn, Random rand, BlockPos position)
    {
        while (worldIn.isAirBlock(position) && position.getY() > 2)
        {
            position = position.down();
        }

        if (worldIn.getState(position).getBlock() != Blocks.snow)
        {
            return false;
        }
        else
        {
            position = position.up(rand.zrange(4));
            int i = rand.zrange(4) + 7;
            int j = i / 4 + rand.zrange(2);

            if (j > 1 && rand.zrange(60) == 0)
            {
                position = position.up(10 + rand.zrange(30));
            }

            for (int k = 0; k < i; ++k)
            {
                float f = (1.0F - (float)k / (float)i) * (float)j;
                int l = ExtMath.ceilf(f);

                for (int i1 = -l; i1 <= l; ++i1)
                {
                    float f1 = (float)ExtMath.absi(i1) - 0.25F;

                    for (int j1 = -l; j1 <= l; ++j1)
                    {
                        float f2 = (float)ExtMath.absi(j1) - 0.25F;

                        if ((i1 == 0 && j1 == 0 || f1 * f1 + f2 * f2 <= f * f) && (i1 != -l && i1 != l && j1 != -l && j1 != l || rand.floatv() <= 0.75F))
                        {
                            Block block = worldIn.getState(position.add(i1, k, j1)).getBlock();

                            if (block.getMaterial() == Material.air || block == Blocks.dirt || block == Blocks.snow || block == Blocks.ice)
                            {
                                this.setBlockAndNotifyAdequately(worldIn, position.add(i1, k, j1), Blocks.packed_ice.getState());
                            }

                            if (k != 0 && l > 1)
                            {
                                block = worldIn.getState(position.add(i1, -k, j1)).getBlock();

                                if (block.getMaterial() == Material.air || block == Blocks.dirt || block == Blocks.snow || block == Blocks.ice)
                                {
                                    this.setBlockAndNotifyAdequately(worldIn, position.add(i1, -k, j1), Blocks.packed_ice.getState());
                                }
                            }
                        }
                    }
                }
            }

            int k1 = j - 1;

            if (k1 < 0)
            {
                k1 = 0;
            }
            else if (k1 > 1)
            {
                k1 = 1;
            }

            for (int l1 = -k1; l1 <= k1; ++l1)
            {
                for (int i2 = -k1; i2 <= k1; ++i2)
                {
                    BlockPos blockpos = position.add(l1, -1, i2);
                    int j2 = 50;

                    if (Math.abs(l1) == 1 && Math.abs(i2) == 1)
                    {
                        j2 = rand.zrange(5);
                    }

                    while (blockpos.getY() > 50)
                    {
                        Block block1 = worldIn.getState(blockpos).getBlock();

                        if (block1.getMaterial() != Material.air && block1 != Blocks.dirt && block1 != Blocks.snow && block1 != Blocks.ice && block1 != Blocks.packed_ice)
                        {
                            break;
                        }

                        this.setBlockAndNotifyAdequately(worldIn, blockpos, Blocks.packed_ice.getState());
                        blockpos = blockpos.down();
                        --j2;

                        if (j2 <= 0)
                        {
                            blockpos = blockpos.down(rand.zrange(5) + 1);
                            j2 = rand.zrange(5);
                        }
                    }
                }
            }

            return true;
        }
    }
}
